#!/usr/bin/perl 

use strict;
use Getopt::Long;
use POSIX;
use File::Basename;
use File::Path;
use DBI;

#--------------------------------------------------
# Setting environment
#-------------------------------------------------- 
$ENV{"USER"}="opuser";
$ENV{"HOME"}="/home/opuser";

#--------------------------------------------------
# Global variables
#--------------------------------------------------
our $name = basename($0, ".pl");
our $version = "2.0";
our $path = "/usr/local/opmon/libexec";
our $temp_log = "$path/$name.log";
our ($opt_help, $opt_verbose, $opt_version, $opt_warn, $opt_crit);
our $conf_file = "/usr/local/opmon/etc/db.php";
our ( $dbuser, $dbpass, $dbhost, $cmd);
our $log_file = "/usr/local/opmon/var/op-discovery.log";

# Database connection info
our $dbh;

# Read database config
_read_conf();
our $database = "opcfg";
our $username = $dbuser;
our $pass = $dbpass;

my $metric = ""; # Generate performance data
my $perf = "$metric=value;$opt_warn;$opt_crit;;"; # Change 'value' to your '$return'

our ($command,$network,$name_xml,$nmap,$nivel_scan,$dir_tmp,$port,$icmp,$user,$method);

$nmap = "sudo /usr/bin/nmap";
$dir_tmp = "/usr/local/opmon/share/seagull/modules/discovery/tmp";

#--------------------------------------------------
# PARA TESTE
#-------------------------------------------------- 
#$network = "192.168.10.212";
#--------------------------------------------------
# Formas de consulta rede
# 192.168.10.212		  => apenas um ip 
# 192.168.10.0\24 ou 192.168.10.* => a rede toda 192.168.10.*
# 192.168.10.100-110		  => faixa de ips do 100 ao 110
# 192.168.10.100,111,112	  => apenas os ips listados	
#
# Formas de consulta portas
# 5666 			=> apenas a porta 5666
# 5666-5668 		=> da porta 5666 até a 5668
# 5666,5667,5669	=> porta 5666,5667 e 5669
#-------------------------------------------------- 
#--------------------------------------------------
# $name_xml = "opmon_vm.xml";
#-------------------------------------------------- 
#$nivel_scan = "5";
#--------------------------------------------------
# Niveis disponiveis
# 0 = Paranoid	=> (para evidar IDS)
# 1 = Sneaky	=> (para evidar IDS)
# 2 = Polite	=> Diminui o ritmo do scan para utilizar menos banda e recursos da maquina alvo
# 3 = Normal	=> Checagem nmap normal
# 4 = Aggressive	=> Acelera o ritmo do scan, devendo ser utilizado em uma rede razoavelmente rapida e confiavel
# 5 = Insane	=> Utilizar em uma rede rapida para nao comprometer a precisao das informacoes
#-------------------------------------------------- 

our ($date_sys);
our ($sec,$min,$hour) = localtime(time);
$date_sys = strftime "%y-%m-%d",localtime;
$date_sys = "20".$date_sys;
our ($ano,$mes,$dia) = split (/-/,$date_sys);
if ($hour == 0){
    $hour = "24";
}

sub main {

	getoption();
	if ($name_xml=="") {
		_logger("ERROR XML name missing",1);
		exit(0);
	}

	$nivel_scan = 5;
	if ($method eq 'AutoDiscovery') {
		$cmd="$nmap -n -sT -O --osscan-limit -oX  $dir_tmp/$name_xml.xml $network -T $nivel_scan -P0";
	}elsif ((defined $port)and(not defined $icmp)){
		$cmd="$nmap -n -sT -O --osscan-limit -oX $dir_tmp/$name_xml.xml $network -T $nivel_scan -p $port -P0";
	}elsif((defined $port) and (defined $icmp)){
		$cmd="$nmap -n -sT -O --osscan-limit -oX $dir_tmp/$name_xml.xml $network -T $nivel_scan -p $port";
	}elsif((not defined $port) and (not defined $icmp)){
		$cmd="$nmap -n -sT -O --osscan-limit -oX $dir_tmp/$name_xml.xml $network -T $nivel_scan -P0";
	}else {
		$cmd="$nmap -n -sT -O --osscan-limit -oX $dir_tmp/$name_xml.xml $network -T $nivel_scan";
	}

	_logger("Running $cmd",1);	
	lock_message();
	$command = `$cmd`;
	status_xml();
	unlink ("$dir_tmp/$name_xml.lock");
}

sub _logger {
	my ($msg, $show) = @_;
	
	my %date = _gettime();
	open(_LOG_, ">>$log_file");
	if ($show==1) {
		print "$date{year}/$date{mon}/$date{mday} $date{hour}:$date{min}:$date{sec} - $msg\n";
	}
	print _LOG_ "$date{year}/$date{mon}/$date{mday} $date{hour}:$date{min}:$date{sec} - $msg\n";
	close(_LOG_);
	
	# Remove old log file if is bigger then 10MB
	my @log_stat = stat($log_file);
	if ($#log_stat>=7) {
		#unlink($log_file) if ($log_stat[7] >= 10485760);
	}
}

#--------------------------------------------------------------------------------------
sub lock_message{
    open (FL,">$dir_tmp/$name_xml.lock");
    print FL "start_job=$ano-$mes-$dia $hour:$min:$sec\n";
    print FL "method=$method\n";
    print FL "user=$user\n";
    print FL "ip_range=$network\n";
    print FL "port_range=$port\n" if ($port);
    print FL "scan_level=$nivel_scan\n";
    print FL "icmp_check=$icmp\n" if ($icmp);
    close (FL);
}

#--------------------------------------------------------------------------------------
sub status_xml {
	open (FL,">>$dir_tmp/$name_xml.inf");
	if ($port){
		print FL "start_job=$ano-$mes-$dia $hour:$min:$sec|method=$method|user=$user|ip_range=$network|scan_level=$nivel_scan|icmp_check=$icmp|port_range=$port";
	}else{
		print FL "start_job=$ano-$mes-$dia $hour:$min:$sec|method=$method|user=$user|ip_range=$network|scan_level=$nivel_scan|icmp_check=$icmp";
	}
	close(FL);
}

#--------------------------------------------------------------------------------------
sub getoption  {
     Getopt::Long::Configure('bundling');
     GetOptions(
	    'm|method=s'              => \$method,
	    'p|port=s'              => \$port,
	    'n|network=s'                  => \$network,
	    'l|level=i'               => \$nivel_scan,
	    'i|icmp=i'               => \$icmp,
	    'x|xmlname=s'               => \$name_xml,
	    'u|user=s'               => \$user,
	    'v|verbose=i'               => \$opt_verbose,
	    'V|version'               => \$opt_version,
	);

     if($opt_help){
	     printHelp();
	      exit(1);
     }

     if($opt_version){
	     print "$name.pl - '$version'\n";
	     exit(1);
     }

     if($opt_verbose){
	     $opt_verbose = 1;
     } else {
	     $opt_verbose = 0;
     }

     # Validate network
     $network = validateNetwork($network);

     # Ignore list
     $network = ignoreList($network);
	  
     # Set ports
     if(!$port) {
			$port = "22,25,80,110,137,138,139,143,389,443,445,689,993,995,3128,5666,5667,8080";
     }

#--------------------------------------------------
#      if ((!$network) or (!$name_xml) or (!$nivel_scan)){
# 
#              printUsage();
#               exit(1);
# 
#      }
#-------------------------------------------------- 
}

#--------------------------------------------------------------------------------------
sub printUsage {

       print <<EOB

Usage: $name.pl [OPTION]...

	-V, --version      Show version
	-m, --method       Metodo utilizado
	-p, --port         Porta para consulta
	-n, --network      Rede a ser consultada
	-l, --level        Nivel de consulta
	-i, --icmp         Habilita consulta icmp
	-u, --user         Usuario que executou
	-x, --xmlname      Nome do xml a ser gerado
	-v, --verbose      0 = Print out in screen
			   1 = Send log to file

EOB

}
#--------------------------------------------------------------------------------------

sub printHelp {

		my $help = <<'HELP';


		Insert help program


HELP

		print $help;

}
#--------------------------------------------------------------------------------------

sub validateNetwork {
	my $network = shift;

	# Remove slashes
	$network =~ s/\// /g;

	# Remove double blank spaces
	$network =~s/\s{2,}/ /g;

	return $network;
}
#--------------------------------------------------------------------------------------

sub ignoreList {
	my $network = shift;
	
	my $query = "SELECT _ignored_objects_list FROM nagios_main";
	$dbh = DBI->connect ( "dbi:mysql:dbname=opcfg;host=$dbhost", "$dbuser", "$dbpass") || die "Failed to get ignore list from opcfg db: $DBI::errstr";
	#my $dbh = DBI->connect('DBI:mysql:opcfg','root') || die "Failed to get ignore list from OPCFG: $DBI::errstr";
	my $sth = $dbh->prepare($query);
	$sth->execute();
	my @ignoreList = $sth->fetchrow_array();
	$sth->finish();
	$dbh->disconnect();

	@ignoreList = split(/\s|\n/,$ignoreList[0]);
	my %hash = ();
	
	foreach(@ignoreList) {
		$hash{$_} = 1;
	}	

	my @arr;
	my @network = split(' ',$network);
	foreach(@network) {
		if(!%hash->{$_} == 1) {
			push(@arr,$_);
		}
	}
	return join(' ',@arr);
}

sub _read_conf {
	open( FILE, "$conf_file" );
	while(<FILE>) {
	  $dbuser = $1 if (m/DBUSER\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
	  $dbpass = $1 if (m/DBPASS\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
	  $dbhost = $1 if (m/DBHOST\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
	}
	close(FILE);
}

sub _gettime {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   $mon++;
   $year+=1900;

   my %self = (
      'wday'	=>	$wday,
      'year'	=>	$year,
      'mon'	=>	$mon,
      'mday'	=>	$mday,
      'hour'	=>	$hour,
      'min'	=>	$min,
      'sec'	=>	$sec
   );
   return %self;
}

#--------------------------------------------------------------------------------------
&main


